/*
  Kaleidoscope
  
  Extract a circular sector from an image and reproduce it several times to crea a kaleidoscope effect
 
  2012-01-26  Pierre Rossel  Initial version
  2023-03-07  Pierre Rossel  Fix for Processing v3 and v4
 
*/

import java.awt.Point;

// source image 
PImage img;

int nFaces = 12; // Number of faces (multiple of 2)

Point ptCenter = new Point(200, 200); // sector center in source image
int iRadius = 180;                  // sector radius
float fAngle = TWO_PI / nFaces;     // sector angle
float fRotation = 0;                // sector rotation in source image


void setup() {
  size(400, 800, P2D);
  img = loadImage("kaleidoscope.jpg");
}

void draw() {
  background(0);

  fAngle = TWO_PI / nFaces;

  pushMatrix();
  {
    translate(0, height / 2);
    image(img, 0, 0);
    //ellipse(ptCenter.x, ptCenter.y, 10, 10); 

    // sector outline
    noFill();
    stroke(255);
    translate(ptCenter.x, ptCenter.y);
    rotate(fRotation);
    arc(0, 0, 2*iRadius, 2*iRadius, 0, fAngle);
    line(0, 0, iRadius, 0);
    rotate(fAngle);
    line(0, 0, iRadius, 0);
  }
  popMatrix();
  text("UP/DOWN/LEFT/RIGHT/CLICK", 2, 398);

  // Create the sector as a mask
  int iWidth = iRadius;
  int iHeight = ceil(iRadius * sin(fAngle));
  PGraphics pgMask = createGraphics(iWidth, iHeight, P2D);
  pgMask.beginDraw();
  pgMask.background(0);
  pgMask.fill(255);
  pgMask.stroke(255);
  pgMask.smooth();
  pgMask.arc(0, 0, 2*iRadius, 2*iRadius, 0, fAngle + 0.008);
  pgMask.rotate(fAngle);
  pgMask.line(0, 0, iRadius, 0);
  pgMask.endDraw();

  // Extract sector from source image
  PGraphics pgTriangle = createGraphics(iWidth, iHeight, P2D);
  pgTriangle.beginDraw();
  pgTriangle.fill(100, 0, 0);
  pgTriangle.rotate(-fRotation);
  pgTriangle.translate(-ptCenter.x, -ptCenter.y);
  pgTriangle.image(img, 0, 0);
  pgTriangle.mask(pgMask);
  pgTriangle.endDraw();

  // draw sectors
  smooth();
  pushMatrix();
  {
    translate(width/2, height/4);
    for (int i = 0; i < nFaces/2; i++) {
      image(pgTriangle, 0, 0);
      scale(1, -1);
      image(pgTriangle, 0, 0);
      scale(1, -1);
      rotate(2 * fAngle);
    }
  }
  popMatrix();
}

void mousePressed() {
  ptCenter.x = mouseX;
  ptCenter.y = mouseY - height / 2;
}

void mouseDragged() {
  mousePressed();
}

void keyPressed() {
  if (keyCode == LEFT) {
    fRotation -= PI / 180;
  }
  else if (keyCode == RIGHT) {
    fRotation += PI / 180;
  }
  else if (keyCode == UP) {
    nFaces += 2;
  }
  else if (keyCode == DOWN) {
    if (nFaces > 4)
      nFaces -= 2;
  }
}
